<?php

namespace SGalinski\SgNews\Service\Backend;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Backend\Template\Components\ButtonBar;
use TYPO3\CMS\Backend\Template\Components\DocHeaderComponent;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Core\Database\QueryGenerator;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Imaging\IconFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Request;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Class Utility
 */
class Utility {
	const CATEGORY_DOKTYPE = 117;
	const NEWS_DOKTYPE = 117;

	/**
	 * create buttons for the backend module header
	 *
	 * @param $docHeaderComponent DocHeaderComponent
	 * @param $request Request
	 * @throws \InvalidArgumentException
	 * @throws \UnexpectedValueException
	 */
	public static function makeButtons(&$docHeaderComponent, &$request) {
		/** @var ButtonBar $buttonBar */
		$buttonBar = $docHeaderComponent->getButtonBar();

		/** @var IconFactory $iconFactory */
		$iconFactory = GeneralUtility::makeInstance(IconFactory::class);

		// Refresh
		$refreshButton = $buttonBar->makeLinkButton()
			->setHref(GeneralUtility::getIndpEnv('REQUEST_URI'))
			->setTitle(LocalizationUtility::translate('LLL:EXT:lang/locallang_core.xlf:labels.reload', ''))
			->setIcon($iconFactory->getIcon('actions-refresh', Icon::SIZE_SMALL));
		$buttonBar->addButton($refreshButton, ButtonBar::BUTTON_POSITION_RIGHT);

		// shortcut button
		$shortcutButton = $buttonBar->makeShortcutButton()
			->setModuleName($request->getPluginName())
			->setGetVariables(
				[
					'id',
					'M'
				]
			)
			->setSetVariables([]);

		$buttonBar->addButton($shortcutButton, ButtonBar::BUTTON_POSITION_RIGHT);
		$docHeaderComponent->getButtonBar();
	}

	/**
	 * Retrieves the next site root in the page hierarchy from the current page
	 *
	 * @param int $currentPid
	 * @return int
	 */
	public static function getSiteRoot($currentPid) {
		$rootLine = BackendUtility::BEgetRootLine((int) $currentPid);
		$siteRoot = ['uid' => 0];

		foreach ($rootLine as $page) {
			if ($page['is_siteroot'] === '1') {
				$siteRoot = $page;
				break;
			}
		}

		return $siteRoot['uid'];
	}

	/**
	 * Get an array of all category uids => >titles below the given site root id
	 *
	 * @param $siteRootPid
	 * @throws \InvalidArgumentException
	 * @return array
	 */
	public static function getCategoriesForSiteRoot($siteRootPid) {
		// get all pageids below the given siteroot
		$queryGenerator = GeneralUtility::makeInstance(QueryGenerator::class);
		$childPids = $queryGenerator->getTreeList($siteRootPid, PHP_INT_MAX, 0, 1);

		// if doktype = 117 (category) then get the category name (page title)
		/** @var DatabaseConnection $databaseConnection */
		$databaseConnection = $GLOBALS['TYPO3_DB'];

		$where = 'doktype = ' . self::CATEGORY_DOKTYPE . ' AND uid in (' . $childPids . ')';
		$result = $databaseConnection->exec_SELECTquery('uid, title', 'pages', $where)->fetch_all();
		$categories = [];
		/** @var array $result */
		foreach ($result as $item) {
			$categories[$item[0]] = $item[1];
		}

		return $categories;
	}
}
